;$MOD252

DA0             BIT P1.0
DA1             BIT P1.1
DA2             BIT P1.2
CS0             BIT P1.3
CS1             BIT P1.4
DIOR            BIT P1.5
DIOW            BIT P1.6
IORDY           BIT P3.1
INTRQ           BIT P3.2
RESET           BIT P3.3
;PLAY           BIT P3.4
;STOP           BIT P3.5
;REV            BIT P3.6
;FWD            BIT P3.7
;PAUSE          BIT P1.7
DDL             EQU P0
DDH             EQU P2

DatabusL        EQU 050H
DatabusH        EQU 051H
Byte_Count      EQU 054H

Audio_Status    EQU 055H       ;Sub-Q Channel Current Audio Status
Track           EQU 056H       ;Current Track Number
Total_Tracks    EQU 058H       ;Track to scan to
Button_State    EQU 059H       ;Last state of the buttons => Port 3
Button_State1   EQU 060H       ;Last state of the buttons => Port 1
M_Field         EQU 061H       ;Minute Field
S_Field         EQU 062H       ;Second Field
F_Field         EQU 063H       ;Frame Field
TOC_Length      EQU 064H       ;TOC data length in bytes
First_Track     EQU 065H       ;First track number on CD
Last_Track      EQU 066H       ;Last track number on CD

               CALL    Initialise
               CALL    Start_Unit
   
loop:          CALL    Get_Buttons
               
               JMP     loop

Initialise:
       MOV     P1,#000H        ;Set Port 1 to zero (needs to be low for internal pull-up for pause)
       MOV     P3,#000001111B  ;Set Button inputs to zero
       SETB    DIOR
       SETB    DIOW
Hard_Reset:
       CLR     RESET           ;Assert Reset-
       CALL    DELAY
       CALL    DELAY
       SETB    RESET           ;Negate Reset-  
       CALL    DELAY
       CALL    DELAY
Soft_Reset:
       CALL    Set_Status  
       CALL    Read
       MOV     A,DatabusL
       ANL     A,#010001000B
       JNZ     Soft_Reset      ;Wait for BSY=0, DRQ=0
       CALL    Set_Command
       MOV     DDL,#008h       ;Soft Reset
       MOV     DDH,#000h
       CALL    Write
       CALL    DELAY
       RET    

Get_Buttons:
       MOV     A,P3
       ANL     A,#011110000B
       CJNE    A,Button_State,but      ;Only scan buttons if buttons have changed
       MOV     A,P1
       ANL     A,#010000000B
       CJNE    A,Button_State1,but
       RET
but:  CALL    Sub_Channel              ;Read current Audio Status and Track Position
       MOV     A,P3   
       ANL     A,#011110000B
       MOV     Button_State,A
       MOV     A,P1
       ANL     A,#010000000B           ;Pause button is on the P1.7
       MOV     Button_State1,A
b0:   MOV     A,P3
       ANL     A,#000010000B
       CJNE    A,#000010000B,b1
       CALL    Play_Button
       RET
b1:   MOV     A,P3
       ANL     A,#000100000B
       CJNE    A,#000100000B,b2
       CALL    Stop_Button
       RET
b2:   MOV     A,P3
       ANL     A,#001000000B
       CJNE    A,#001000000B,b3
       CALL    Rev_Button
       RET
b3:   MOV     A,P3
       ANL     A,#010000000B
       CJNE    A,#010000000B,b4
       CALL    Fwd_Button
       RET
b4:   MOV     A,P1
       ANL     A,#010000000B
       CJNE    A,#010000000B,b5
       CALL    Pause_Button
b5:   RET

Play_Button:
                       CALL    Read_TOC
                       MOV     M_Field,#000H
                       MOV     S_Field,#002H
                       MOV     F_Field,#000H
                       CALL    Play
                       RET

Stop_Button:   CALL    Stop_Play
                       RET

Fwd_Button:    MOV     DPH,#00FH   
                       MOV     DPL,#000H   ;Last 256 Bytes of memory
                       MOV     A,Track
                       INC     A
                       MOV     Track,A
 Get_Fwd_MSF:   
                       MOV     A,DPL
                       INC       A
                       MOV       DPL,A
                       MOVX    A,@DPTR
                       MOV     M_Field,A
                       MOV     A,DPL
                       INC       A
                       MOV       DPL,A
                       MOVX    A,@DPTR
                       MOV     S_Field,A
                       MOV     A,DPL
                       INC       A
                       MOV       DPL,A
                       MOVX    A,@DPTR
                       MOV     F_Field,A
                       MOV     A,Track
                       DEC     A
                       MOV     Track,A
                       MOV     A,Track
                       JNZ     Get_Fwd_MSF     ;Loop until desired track reached           
                       CALL    Play   
                       RET

Rev_Button:    MOV       DPH,#00FH   ;
                       MOV     DPL,#000H   ;Last 256 Bytes of memory
                       MOV     A,Track
                       DEC     A
                       MOV     Track,A

  Get_Rev_MSF:         MOV     A,DPL
                       INC       A
                       MOV       DPL,A
                       MOVX    A,@DPTR
                       MOV     M_Field,A
                       MOV     A,DPL
                       INC       A
                       MOV       DPL,A
                       MOVX    A,@DPTR
                       MOV     S_Field,A
                       MOV     A,DPL
                       INC       A
                       MOV       DPL,A
                       MOVX    A,@DPTR
                       MOV     F_Field,A
                       MOV     A,Track
                       DEC     A
                       MOV     Track,A
                       MOV     A,Track
                       JNZ     Get_Rev_MSF ;Loop until desired track reached       
                       CALL    Play
                       RET

Pause_Button:
               CALL    Pause
               RET

Send_Packet:
       CALL    Set_Status  
       CALL    Read
       MOV     A,DatabusL
       ANL     A,#010001000B
       JNZ     Send_Packet         ;Wait for BSY=0, DRQ=0
       CALL    Set_DriveSelect
       MOV     DDL,#011100000B     ;Select Drive 0, LBA
       MOV     DDH,#00H
       CALL    Write        
       CALL    Set_ByteCountLow
       MOV     DDL,#0FFH           ;Maximum 255!!!!!       
       CALL    Write
       CALL    Set_ByteCountHigh
       MOV     DDL,#0FFH                ;Maximum 255!!!!!  
       CALL    Write
       CALL    Set_Feature
       CALL    Write
       CALL    Set_Command
       MOV     DDL,#0A0H               ;ATAPI Packet Command
       CALL    Write
Wait_DRQ:
       CALL    Set_Status      
       CALL    Read
       MOV     A,DatabusL
       ANL     A,#010001000B
       CJNE    A,#000001000B,Wait_DRQ  ;Poll for DRQ=1,BSY=0
       RET

Start_Unit:
       CALL    Send_Packet
       CALL    Set_Data
       MOV     DDL,#01BH       ;Operation Code
       MOV     DDH,#000H       ;Reserved, Bit0,Immed = 0
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#001H       ;Reserved, Bit1,LoEj = 0, Bit0,Start = 1
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       CALL    Write
       CALL    Write
       CALL    Wait_INTRQ
       RET

Stop_Play:
       CALL    Send_Packet
       CALL    Set_Data
       MOV     DDL,#04EH       ;Operation Code
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       CALL    Write
       CALL    Write
       CALL    Write
       CALL    Write
       CALL    Write
       CALL    Wait_INTRQ
       RET

Play:   
       CALL    Send_Packet
       CALL    Set_Data
       MOV     DDL,#047H       ;Play Audio MSF Command
       MOV     DDH,#000H       ;Resered
       CALL    Write
       MOV     DDL,#000H       ;Resered
       MOV     A,M_Field
       MOV     DDH,A           ;Starting M Field
       CALL    Write
       MOV     A,S_Field
       MOV     DDL,A           ;Starting S Field
       MOV     A,F_Field
       MOV     DDH,A           ;Starting F Field
       CALL    Write
       MOV     DDL,#063H       ;Ending M Field  
       MOV     DDH,#03BH       ;Ending S Field
       CALL    Write
       MOV     DDL,#04AH       ;Ending F Field
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       CALL    Write
       CALL    Wait_INTRQ       
       RET

Pause:  
       CALL    Send_Packet 
       CALL    Set_Data
       MOV     DDL,#04BH       ;Pause/Resume Command
       MOV     DDH,#000H       ;Resered
       CALL    Write
       MOV     DDL,#000H       ;Resered
       CALL    Write
       CALL    Write
       CALL    Write
       ;Check if already paused
       MOV     DDL,#000H       ;Reserved, Resume Bit(0) (Pause if playing)
       MOV     A,Audio_Status
       CJNE    A,#012H,Paused
       MOV     DDL,#001H       ;Resume if Paused
Paused:     
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       CALL    Write
       CALL    Wait_INTRQ       
       RET

Sub_Channel:    
       CALL    Send_Packet
       CALL    Set_Data
       MOV     DDL,#042H       ;Read Sub-Channel Command
       MOV     DDH,#002H       ;Resered, Bit1=MSF, Bit0=Reserved
       CALL    Write
       MOV     DDL,#040H       ;Resered, Bit6=SubQ, Reserved
       MOV     DDH,#001H       ;Sub-cahnnel Data Format (CD-ROM current position)
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Track number (Must be zero - Not relevant)
       MOV     DDH,#000H       ;Allocation Length (MSB) (!!!!!! I think zero!)
       CALL    Write
       MOV     DDL,#001H       ;Allocation Length (LSB) (!!!!!! I think ONE!)
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       CALL    Wait_INTRQ   
       CALL    Read_Channel_Data   
       RET

Read_TOC:    
       CALL    Send_Packet
       CALL    Set_Data
       MOV     DDL,#043H       ;Read Sub-Channel Command
       MOV     DDH,#002H       ;Resered, Bit1=MSF, Bit0=Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved, Format
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       MOV     DDL,#000H       ;Starting Track
       MOV     DDH,#000H       ;Allocation Length (MSB) 
       CALL    Write
       MOV     DDL,#001H       ;Allocation Length (LSB) 
       MOV     DDH,#000H       ;Format, Reserved
       CALL    Write
       MOV     DDL,#000H       ;Reserved
       MOV     DDH,#000H       ;Reserved
       CALL    Write
       CALL    Wait_INTRQ   
       CALL    Read_TOC_Data   
       RET

Wait_INTRQ:        
       MOV     A,P3
       ANL     A,#000000100B
       JZ      Wait_INTRQ
       CALL    Set_Status
       CALL    Read
       CALL    Set_CauseOfInt      ;To clear the interupt for Data In
       CALL    Read 
       RET

Read_Channel_Data:
       ;Wait for DRQ=1
       CALL    Set_Status
       CALL    Read
       MOV     A,DatabusL
       ANL     A,#000001000B   ;Wait DRQ=1
       JZ      Read_Channel_Data

       CALL    Set_Data
       CALL    Read
       MOV     A,DatabusH       ;Bit 1
       MOV     Audio_Status,A
       CALL    Read
       CALL    Read
       CALL    Read
       MOV     A,DatabusL       ;Bit 6
       MOV     Track,A
       CALL    Read
       CALL    Read
       CALL    Read
       CALL    Read

       ;Should be a DRQ=0 NOW
       CALL    Set_Status
       CALL    Read
       
       RET

Read_TOC_Data:  
       ;Wait for DRQ=1
       CALL    Set_Status
       CALL    Read
       MOV     A,DatabusL
       ANL     A,#000001000B   ;Wait DRQ=1
       JZ      Read_TOC_Data

       CALL    Set_Data
       CALL    Read
       MOV     A,DatabusH       ;Bit 1
       MOV     TOC_Length,A    ;LIMITATION - Only reads ONE byte of the length!
                               ;              IE Less that 255 bytes 36 Tracks MAX 
       CALL    Read
       MOV     A,DatabusL
       MOV     First_Track,A
       MOV     A,DatabusH
       MOV     Last_Track,A
       DEC     TOC_Length
       DEC     TOC_Length

       ;ALLOCATE THE FIRST PORTION OF MEMORY HERE
       MOV     DPH,#00FH   ;
       MOV     DPL,#000H   ;Last 256 Bytes of memory

Next_Descriptor:
       CALL    Read
       CALL    Read    
       CALL    Read
       ;M Field
       MOV     A,DPL
       INC       A
       MOV       DPL,A
       MOV     A,DatabusH
       MOVX    @DPTR,A
       CALL    Read
       ;S Field
       MOV     A,DPL
       INC       A
       MOV       DPL,A
       MOV     A,DatabusL
       MOVX    @DPTR,A 
       ;F Field
       MOV     A,DPL
       INC       A
       MOV       DPL,A
       MOV     A,DatabusH
       MOVX    @DPTR,A 
       DEC     TOC_Length
       DEC     TOC_Length
       DEC     TOC_Length
       DEC     TOC_Length
       DEC     TOC_Length
       DEC     TOC_Length
       DEC     TOC_Length
       DEC     TOC_Length
       MOV     A,TOC_Length
       JNZ     Next_Descriptor
       ;Should be a DRQ=0 NOW

TOC_DRQ_Wait:    
       CALL    Set_Status      
       CALL    Read
       MOV     A,DatabusL
       ANL     A,#000001000B
       CJNE    A,#000000000B,TOC_DRQ_Wait  ;Poll for DRQ=0
       
       RET

Set_Feature:
Set_CauseOfInt:   CLR     CS0     
                       SETB    CS1    
                       CLR     DA2     
                       SETB    DA1    
                       CLR     DA0    
                       RET
Set_ByteCountLow: CLR     CS0
                       SETB    CS1
                       SETB    DA2
                       CLR     DA1
                       CLR     DA0
                       RET
Set_ByteCountHigh:CLR     CS0
                       SETB    CS1
                       SETB    DA2
                       CLR     DA1
                       SETB    DA0
                       RET
Set_DriveSelect:  CLR     CS0     
                       SETB    CS1     
                       SETB    DA2     
                       SETB    DA1     
                       CLR     DA0     
                       RET
Set_Command: 
Set_Status:       CLR     CS0   
                       SETB    CS1    
                       SETB    DA2     
                       SETB    DA1    
                       SETB    DA0    
                       RET
Set_Data:         CLR     CS0
                       SETB    CS1
                       CLR     DA2
                       CLR     DA1
                       CLR     DA0
                       RET

Read:       MOV     DDL,#0FFH       ;Port0 Pull-ups
               CLR     DIOR            ;Assert DIOR
               MOV     A,DDL
               MOV     DatabusL,A
               MOV     A,DDH
               MOV     DatabusH,A
               SETB    DIOR            ;Negate DIOR
               MOV     DDH,#000H
               RET

Write:      NOP
               CLR     DIOW            ;Assert DIOW
               NOP
               SETB    DIOW            ;Negate DIOW
               RET

DELAY:      MOV     R4,#00AH
DELAY1:     MOV     R2,#0A0H
D2:         MOV     R3,#000H
D3:         XCH     A,R5
               XCH     A,R5
               DJNZ    R3,D3
               DJNZ    R2,D2
               DJNZ    R4,DELAY1      
               RET         
END
